@@ -20,15 +20,15 @@ CXX=clang++
 : ${CXX:="g++"}
 : ${FC:="gfortran"}
 
-: ${ENABLE_DPCPP:="ON"}
-: ${ENABLE_OPENCL:="OFF"}
-: ${ENABLE_CUDA:="ON"}
-: ${ENABLE_HIP="OFF"}
-: ${ENABLE_OPENMP="OFF"}
-: ${ENABLE_METAL="OFF"}
-: ${ENABLE_FORTRAN="OFF"}
-: ${ENABLE_TESTS="ON"}
-: ${ENABLE_EXAMPLES="ON"}
+: ${OCCA_ENABLE_DPCPP:="ON"}
+: ${OCCA_ENABLE_OPENCL:="OFF"}
+: ${OCCA_ENABLE_CUDA:="ON"}
+: ${OCCA_ENABLE_HIP="OFF"}
+: ${OCCA_ENABLE_OPENMP="OFF"}
+: ${OCCA_ENABLE_METAL="OFF"}
+: ${OCCA_ENABLE_FORTRAN="OFF"}
+: ${OCCA_ENABLE_TESTS="ON"}
+: ${OCCA_ENABLE_EXAMPLES="ON"}
 
 cmake -S . -B ${BUILD_DIR} \
   -DCMAKE_BUILD_TYPE=${BUILD_TYPE} \
@@ -39,12 +39,12 @@ cmake -S . -B ${BUILD_DIR} \
   -DCMAKE_CXX_FLAGS="${CXXFLAGS}" \
   -DCMAKE_C_FLAGS="${CFLAGS}" \
   -DCMAKE_Fortran_FLAGS="${FFLAGS}" \
-  -DENABLE_OPENMP=${ENABLE_OPENMP} \
-  -DENABLE_OPENCL=${ENABLE_OPENCL} \
-  -DENABLE_DPCPP=${ENABLE_DPCPP} \
-  -DENABLE_CUDA=${ENABLE_CUDA} \
-  -DENABLE_HIP=${ENABLE_HIP} \
-  -DENABLE_METAL=${ENABLE_METAL} \
-  -DENABLE_FORTRAN=${ENABLE_FORTRAN} \
-  -DENABLE_TESTS=${ENABLE_TESTS} \
-  -DENABLE_EXAMPLES=${ENABLE_EXAMPLES}
+  -DOCCA_ENABLE_OPENMP=${OCCA_ENABLE_OPENMP} \
+  -DOCCA_ENABLE_OPENCL=${OCCA_ENABLE_OPENCL} \
+  -DOCCA_ENABLE_DPCPP=${OCCA_ENABLE_DPCPP} \
+  -DOCCA_ENABLE_CUDA=${OCCA_ENABLE_CUDA} \
+  -DOCCA_ENABLE_HIP=${OCCA_ENABLE_HIP} \
+  -DOCCA_ENABLE_METAL=${OCCA_ENABLE_METAL} \
+  -DOCCA_ENABLE_FORTRAN=${OCCA_ENABLE_FORTRAN} \
+  -DOCCA_ENABLE_TESTS=${OCCA_ENABLE_TESTS} \
+  -DOCCA_ENABLE_EXAMPLES=${OCCA_ENABLE_EXAMPLES}
