@@ -19,8 +19,9 @@
 #    include <errno.h>
 #    include <sys/sysinfo.h>
 #  else // OCCA_MACOS_OS
+#    include <AvailabilityMacros.h>
 #    include <mach/mach_host.h>
-#    ifdef __clang__
+#    if MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
 #      include <CoreServices/CoreServices.h>
 #      include <mach/mach_time.h>
 #    else
@@ -82,7 +83,7 @@ namespace occa {
 
       return (double) (ct.tv_sec + (1.0e-9 * ct.tv_nsec));
 #elif (OCCA_OS == OCCA_MACOS_OS)
-#  if defined __clang__ && defined CLOCK_UPTIME_RAW
+#  if MAC_OS_X_VERSION_MIN_REQUIRED >= 101200
       uint64_t nanoseconds = clock_gettime_nsec_np(CLOCK_UPTIME_RAW);
 
       return 1.0e-9 * nanoseconds;
@@ -401,9 +402,9 @@ namespace occa {
 
     int getTID() {
 #if (OCCA_OS & (OCCA_LINUX_OS | OCCA_MACOS_OS))
-#if OCCA_OS == OCCA_MACOS_OS & (MAC_OS_X_VERSION_MAX_ALLOWED >= 101200)
+#if OCCA_OS == OCCA_MACOS_OS & (MAC_OS_X_VERSION_MIN_REQUIRED >= 1070)
       uint64_t tid64;
-      pthread_threadid_np(NULL, &tid64);
+      pthread_threadid_np(nullptr, &tid64);
       pid_t tid = (pid_t)tid64;
 #else
       pid_t tid = syscall(SYS_gettid);
@@ -992,7 +993,7 @@ namespace occa {
     }
 
     void runFunction(functionPtr_t f, const int argc, void **args) {
-#include "runFunction.cpp_codegen"
+#include "codegen/runFunction.cpp_codegen"
     }
 
     std::string stacktrace(const int frameStart,
